@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package Net::SAML2::Util;
-our $VERSION = '0.77'; # VERSION
+our $VERSION = '0.78'; # VERSION
 
 use Crypt::OpenSSL::Random qw(random_pseudo_bytes);
 
@@ -11,12 +11,17 @@ use Exporter qw(import);
 
 our @EXPORT_OK = qw(
     generate_id
+    deprecation_warning
 );
 
 sub generate_id {
     return 'NETSAML2_' . unpack 'H*', random_pseudo_bytes(32);
 }
 
+sub deprecation_warning {
+    warn "Net::SAML2 deprecation warning: " . shift . "\n";
+}
+
 
 1;
 
@@ -32,7 +37,7 @@ Net::SAML2::Util - Utility functions for Net::SAML2
 
 =head1 VERSION
 
-version 0.77
+version 0.78
 
 =head1 SYNOPSIS
 
@@ -46,6 +51,10 @@ version 0.77
 
 Generate a NETSAML2 Request Id
 
+=head2 sub deprecation_warning {}
+
+Show a warning that a Deprecated feature is being used
+
 =head1 AUTHORS
 
 =over 4
